/*
*************************************************************************************************
_____________________________________________________
*/
/*
 * Simple demo program that talks to a 24Cxx IC EEPROM using the
 * builtin TWI interface of an ATmega device.
 */

#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <avr/io.h>
#include <compat/twi.h>         /* Note [1] */
#define F_CPU 16000000UL        /* Note [2] */
#include <util/delay.h>
#include <TWI.c>
#include "uart.c"
//#define DEBUG 1

/*
 * System clock in Hz.
 */


// Konstanten

unsigned const char SCL = 0;				//
unsigned const char SDA = 1;				//
unsigned const char DDRY = 2;				// 
unsigned const char A0 = 3;				// 
unsigned const char RFS = 4;				// 
unsigned const char TFS = 5;				// Steuerbits fr AD-Wandler


unsigned long Erg = 0;
double Mittelwert = 0;
unsigned long TX_word = 0;
unsigned long RX_word = 0;
unsigned char count = 0;	
//****************************************************************************************
void AD_setup (void)
{
	DDRC |= (1<<SCL)|(1<<SDA)|(1<<A0);
	PORTC &= ~(1<<SDA)&~(1<<SCL);
	PORTC &= ~(1<<A0);
	PORTC |= (1<<RFS);
	count = 0;
	TX_word = 0b000000000000001100000000; // 24-bit-Modus, RTD EIN
	PORTC &= ~(1<<TFS);
	
TX_next:	
	if (count >= 24)
	{
		goto TX_ENDE;
	}
		if ((TX_word & 1) == 1)
			{
				PORTC |= (1<<SDA);
			}
			else
			{
				PORTC &= ~(1<<SDA);
			}
		PORTC |= (1<<SCL);
		_delay_us(5);
		TX_word >>= 1;
		PORTC &= ~(1<<SCL);
		_delay_us(5);
		count++;
		goto TX_next;
TX_ENDE:

	PORTC |= (1<<RFS)|(1<<TFS);
	PORTC |= (1<<A0);
	DDRC &= ~(1<<SDA);
}	
/*
 * Compatibility defines.  This should work on ATmega8, ATmega16,
 * ATmega163, ATmega323 and ATmega128 (IOW: on all devices that
 * provide a builtin TWI interface).
 *
 * On the 128, it defaults to USART 1.
 */


void ioinit(void)
{

DDRC &= ~(1<<DDRY);						// DDRY auf Eingang
DDRC |= (1<<SCL)|(1<<A0)|(1<<RFS)|(1<<TFS);					// A0 und RFS und TFS auf Ausgang
PORTC |= (1<<DDRY);						// Pullup fr DDRY

uart_init();
}

/*
 * Note [6]
 * Send character c down the UART Tx, wait until tx holding register
 * is empty.
 */


FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//****************************************************************************************
//****************************************************************************************
//****************************************************************************************

int main(void)
{

stdout = stdin = &uart_str;						// Streams festlegen

  uint8_t x;

  ioinit();
  //TWI_init();
  AD_setup();
  
printf ("AD7711 - Test v0.9\n");
  
  Mittelwert = 0;
  Erg = 0;
  
PORTC |= (1<<RFS)|(1<<TFS)|(1<<A0);

	DDRC |= (1<<SCL);
	DDRC &= ~(1<<SDA);
	PORTC &= ~(1<<SCL);
	
Neustart:
	
	_delay_ms(35);
	
	while ((PINC &(1<<DDRY)));	
	PORTC &= ~(1<<RFS);

	for (x = 1; x <= 24; x++)
	{
		PORTC &= ~(1<<SCL);
		_delay_us(5);
		if (((PINC &(1<<SDA))))
		{
			Erg ++;
		}
		Erg <<= 1;
		_delay_us(5);
		PORTC |= (1<<SCL);
		_delay_us(5);
	}

	PORTC |= (1<<RFS);
	
	Erg += Erg;
	
	while (count <= 30)
	{
		count++;
		goto Neustart;
	}
	
	Erg ^= 16777215;
	Mittelwert = Erg;
	Mittelwert = Mittelwert / 31;
	Mittelwert = (50000000/16777215)*Mittelwert;
	Mittelwert = Mittelwert / 10000000;
	printf("% 7.6f Volt\n",Mittelwert);
	Mittelwert = 0;
	count = 0;
	
goto Neustart;	
	

}

